from five import grok
from seantis.agencies.browser.base import BaseView
from seantis.agencies.interfaces import IActivityEvent
from seantis.agencies.types import IOrganization
from seantis.plonetools import tools
from zope.event import notify
from zope.interface import implements


class ResourceViewedEvent(object):
    implements(IActivityEvent)


class OrganizationView(BaseView):

    grok.require('zope2.View')
    grok.context(IOrganization)
    grok.name('view')

    template = grok.PageTemplateFile('templates/organization.pt')
    event_fired = False

    def suborganizations(self):
        return [
            (brain["Title"] if "Title" in brain else "", brain.getURL())
            for brain in self.context.suborganizations()
        ]

    def memberships(self):
        memberships = []
        for brain in self.context.memberships():
            obj = brain.getObject()
            person = obj.person.to_object
            name = person.title if person else u''
            memberships.append((obj.role, name, obj.prefix, brain.getURL()))

        if self.context.display_alphabetically:
            sortkey = lambda m: tools.unicode_collate_sortkey()(m[1])
            memberships = sorted(memberships, key=sortkey)

        return memberships

    def update(self, **kwargs):
        super(OrganizationView, self).update(**kwargs)

        if not self.event_fired:
            notify(ResourceViewedEvent())
            self.event_fired = True



from Acquisition import aq_inner, aq_parent
from collections import namedtuple
from five import grok
from plone import api
from seantis.agencies.types import IMember
from seantis.people.browser.macros import View as BaseView
from seantis.people.interfaces import IPerson
from seantis.plonetools import tools


class View(BaseView):

    """ Overrides seantis.people.browser.macros to include the parent
    organisation in the membership listing on the person details view.

    """

    grok.context(IMember)
    grok.require('zope2.View')
    grok.name('seantis-people-macros')

    def organizations(self, person, method):
        Organization = namedtuple('Organization', ['title', 'url', 'role'])
        catalog = api.portal.get_tool('portal_catalog')

        organizations = []

        for uuid, memberships in getattr(person, method).items():

            current_role = IPerson(person).current_role(memberships)

            brain = catalog(UID=uuid)[0]
            obj = brain.getObject()
            parent = aq_parent(aq_inner(obj))

            title = u'%s' % obj.title
            if parent:
                title = u'%s (%s)' % (title, parent.title)

            organizations.append(
                Organization(title, brain.getURL(), current_role)
            )

        sortkey = lambda org: tools.unicode_collate_sortkey()(org.title)
        return sorted(organizations, key=sortkey)



from bleach import linkify
from plone import api

from seantis.agencies.types import IMembership


def on_organization_modified(context, event=None):
    """ Listens to IObjectModifiedEvent events of organization and triggers
    reindexing for containing memberships.

    """
    if event.descriptions:
        attributes = []
        for attribute in event.descriptions:
            attributes.extend(attribute.attributes)
        if 'title' in attributes:
            idxs = (
                "seantis.people.standard_selectable_organizations",
                "seantis.agencies.member_selectable_organizations"
            )

            # memberships are in the normal catalog!
            catalog = api.portal.get_tool('portal_catalog')
            memberships = catalog(
                object_provides=IMembership.__identifier__,
                path={'query': '/'.join(context.getPhysicalPath()), 'depth': 1}
            )
            for membership in memberships:
                person = membership.getObject().person.to_object
                if person:
                    person.reindexObject(idxs=idxs)
        if 'portrait' in attributes:
            callbacks = []
            context.portrait = linkify(
                context.portrait, callbacks=callbacks, parse_email=True
            )



from five import grok
from seantis.agencies.browser.base import BaseView
from seantis.agencies.types import IMembership


class OrganizationView(BaseView):

    grok.require('zope2.View')
    grok.context(IMembership)
    grok.name('view')

    template = grok.PageTemplateFile('templates/membership.pt')

    def person(self):
        person = self.context.person.to_object
        if person:
            return [person.title, person.absolute_url()]
        return [u'', u'']



from five import grok
from seantis.agencies.interfaces import ISeantisAgenciesSpecific
from seantis.plonetools.browser import BaseForm as SharedBaseForm
from seantis.plonetools.browser import BaseView as SharedBaseView


class BaseView(SharedBaseView):

    grok.baseclass()
    grok.layer(ISeantisAgenciesSpecific)

    domain = 'seantis.agencies'


class BaseForm(SharedBaseForm):

    grok.baseclass()
    grok.layer(ISeantisAgenciesSpecific)

    domain = 'seantis.agencies'



import logging

from collective.dexteritytextindexer import searchable
from plone import api
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation
from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
from plone.dexterity.content import Container
from plone.directives import form
from plone.namedfile.field import NamedImage
from seantis.agencies import _
from seantis.people.content import Membership as PeopleMembership
from seantis.people.interfaces import IMembership as IPeopleMembership
from seantis.people.types.base import PersonBase
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


log = logging.getLogger('seantis.agencies')


class IMember(form.Schema):
    form.model("member.xml")


class Member(PersonBase):
    pass


@form.default_value(field=IExcludeFromNavigation['exclude_from_nav'])
def excludeFromNavDefaultValue(data):
    return True


class IOrganization(form.Schema):

    searchable('title')
    title = schema.TextLine(
        title=_(u'Title')
    )

    searchable('description')
    description = schema.Text(
        title=_(u'Description'),
        required=False
    )

    searchable('portrait')
    form.widget(portrait=WysiwygFieldWidget)
    portrait = schema.Text(
        title=_(u'Portrait'),
        required=False
    )

    display_alphabetically = schema.Bool(
        title=_(u'Display memberships alphabetically'),
        description=_(
            u'Tick this box to sort memberships alphabetically by name rather '
            u'than by their position in the folder.'
        ),
        default=False
    )

    export_fields = schema.List(
        title=_(u"Fields to export"),
        description=_(u"Fields to include in the PDF export"),
        required=False,
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary(terms=[
                SimpleTerm(value=u'role', title=_('Role')),
                SimpleTerm(value=u'title', title=_('Last Name First Name')),
                SimpleTerm(value=u'start', title=_('Start of membership')),
                SimpleTerm(value=u'postfix', title=_('Postfix')),
                SimpleTerm(value=u'lastname', title=_('Last Name')),
                SimpleTerm(value=u'firstname', title=_('First Name')),
                SimpleTerm(value=u'year', title=_('Year')),
                SimpleTerm(value=u'academic_title', title=_('Academic Title')),
                SimpleTerm(value=u'occupation', title=_('Occupation')),
                SimpleTerm(value=u'address', title=_('Address')),
                SimpleTerm(value=u'political_party',
                           title=_('Political Party')),
                SimpleTerm(value=u'phone', title=_('Phone')),
                SimpleTerm(value=u'direct_number', title=_('Direct number')),
            ])
        ),
        default=['role', 'title']
    )

    organigram = NamedImage(
        title=_(u'Organigram'),
        required=False
    )


class Organization(Container):

    pdfexportable = True

    def memberships(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.getPhysicalPath())

        memberships = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.membership',
            sort_on='getObjPositionInParent',
        )

        return memberships

    def suborganizations(self):
        catalog = api.portal.get_tool('portal_catalog')
        folder_path = '/'.join(self.getPhysicalPath())

        suborganizations = catalog(
            path={'query': folder_path, 'depth': 1},
            portal_type='seantis.agencies.organization',
            sort_on='getObjPositionInParent',
        )

        return suborganizations


class IMembership(IPeopleMembership):

    start = schema.TextLine(
        title=_(u"Start of membership"),
        required=False
    )

    prefix = schema.TextLine(
        title=_(u"Prefix"),
        required=False
    )

    postfix = schema.TextLine(
        title=_(u"Postfix"),
        required=False
    )


class Membership(PeopleMembership):
    pass



from zope.interface import Interface


class ISeantisAgenciesSpecific(Interface):
    """ Browser Layer for seantis.agencies. """


class IActivityEvent(Interface):
    """ Event triggered when a seantis.agencies.organization resource
    is first viewed."""



import xlrd

from copy import deepcopy
from seantis.agencies import tests
from seantis.agencies.browser.xlsexport import TITLES_ORGANIZATION
from seantis.agencies.browser.xlsexport import TITLES_REGISTER
from StringIO import StringIO
from xlwt import Workbook
from zExceptions import Unauthorized


class TestImportExport(tests.BrowserTestCase):

    def setUp(self):
        super(TestImportExport, self).setUp()
        self.organizations = [
            [u'0', u'1,2', u'Organizations', u'des_0', u'port_0', 0],
            [u'1', u'', u'org_1', u'des_1', u'port_1', 0],
            [u'2', u'3', u'org_2', u'des_2', u'port_2', 0],
            [u'3', u'', u'org_3', u'des_3', u'port_3', 1]
        ]
        self.people = [
            [str(prefix) + '_' + field for field in TITLES_REGISTER]
            for prefix in range(3)
        ]
        self.people[0][-1] = u'(0)(role)(2006)(*)(3)//'
        self.people[1][-1] = u'(1)()()()(7)// (0)()()()(2)//(3)()()()(7)'
        self.people[2][-1] = u'(2)(role)()(2000)(2)//(3)()(*)()(8)'

    def create_test_workbook(self, filehandle, organizations=None,
                             people=None):
        if organizations is None:
            organizations = self.organizations
        if people is None:
            people = self.people

        book = Workbook(encoding='utf-8')
        sheet = book.add_sheet(u'Organizations')
        for index, cell in enumerate(TITLES_ORGANIZATION):
            sheet.row(0).write(index, cell)
        for row, organization in enumerate(organizations):
            for index, cell in enumerate(organization):
                sheet.row(row + 1).write(index, cell)
        sheet = book.add_sheet(u'People')
        for index, cell in enumerate(TITLES_REGISTER):
            sheet.row(0).write(index, cell)
        for row, organization in enumerate(people):
            for index, cell in enumerate(organization):
                sheet.row(row + 1).write(index, cell)
        book.save(filehandle)

    def test_access(self):
        browser = self.new_browser()
        self.assertRaises(Unauthorized, browser.open, '/export-agencies')
        self.assertRaises(Unauthorized, browser.open, '/import-agencies')

    def test_export_empty(self):
        browser = self.new_admin_browser()
        browser.open('/export-agencies')
        self.assertEquals(browser.contents, '')
        self.assertEquals(browser.headers['Content-disposition'], 'export.xls')
        self.assertEquals(browser.headers['Content-Type'], 'application/xls')
        self.assertEquals(browser.headers['Content-Length'], '0')

    def test_import_export(self):
        filehandle = StringIO()
        self.create_test_workbook(filehandle)
        xls_file = filehandle.getvalue()
        filehandle.close()

        # import
        browser = self.new_admin_browser()
        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()

        # export
        browser.open('/export-agencies')
        self.assertEquals(browser.headers['Content-disposition'], 'export.xls')
        self.assertEquals(browser.headers['Content-Type'], 'application/xls')
        self.assertTrue(int(browser.headers['Content-Length']) > 0)

        # compare
        workbook = xlrd.open_workbook(file_contents=browser.contents)
        sheets = workbook.sheets()
        self.assertEquals(len(sheets), 2)
        self.assertEquals(sheets[0].name, u'Organizations')
        self.assertEquals(sheets[0].ncols, len(TITLES_ORGANIZATION))
        self.assertEquals(sheets[0].nrows, 5)

        for row, content in enumerate(self.organizations):
            self.assertEquals([cell.value for cell in sheets[0].row(row + 1)],
                              content)

        self.assertEquals(sheets[1].name, u'People')
        self.assertEquals(sheets[1].ncols, len(TITLES_REGISTER))
        self.assertEquals(sheets[1].nrows, 4)
        for row, content in enumerate(self.people):
            self.assertEquals(
                [cell.value for cell in sheets[1].row(row + 1)[:-1]],
                content[:-1]
            )

        # roles are slightly different: position starts at zero, order is
        # random
        self.assertEquals(u'(0)(role)(2006)(*)(1)',
                          sheets[1].cell(1, -1).value)
        self.assertTrue(u'(1)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(3)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(0)()()()(0)', sheets[1].cell(2, -1).value)
        self.assertTrue(u'(2)(role)()()(0)', sheets[1].cell(3, -1).value)
        self.assertTrue(u'(3)()()(*)(1)', sheets[1].cell(3, -1).value)

        # browse imported items
        browser.open('/organizations')
        self.assertIn('des_0', browser.contents)
        self.assertIn('port_0', browser.contents)
        self.assertIn('org_1', browser.contents)
        self.assertIn('org_2', browser.contents)
        self.assertNotIn('org_3', browser.contents)

        browser.open('/organizations/org_1')
        self.assertIn('des_1', browser.contents)
        self.assertIn('port_1', browser.contents)
        self.assertNotIn('org_2', browser.contents)
        self.assertNotIn('org_3', browser.contents)

        browser.open('/organizations/org_2')
        self.assertIn('des_2', browser.contents)
        self.assertIn('port_2', browser.contents)
        self.assertNotIn('org_1', browser.contents)
        self.assertIn('org_3', browser.contents)

        browser.open('/organizations/org_2/org_3')
        self.assertIn('des_3', browser.contents)
        self.assertIn('port_3', browser.contents)

    def test_import_errors(self):
        browser = self.new_admin_browser()

        # missing file
        browser.open('/import-agencies')
        browser.getControl('Import').click()
        self.assertIn('Required input is missing', browser.contents)

        # invalid file
        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file('stuff', 'application/xls', 'import.xls')
        browser.getControl('Import').click()
        self.assertIn('Invalid XLS file', browser.contents)

        # error in register sheet
        people = deepcopy(self.people)
        people[1][-1] = u'(1))()()(7)// (0aa()()(2)//(3)()()()(7)//'
        filehandle = StringIO()
        self.create_test_workbook(filehandle, people=people)
        xls_file = filehandle.getvalue()
        filehandle.close()

        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()

        self.assertIn('Invalid row 2 in sheet People', browser.contents)

        # error in organization sheet
        organizations = deepcopy(self.organizations)
        organizations[0][2] = u'Urganizations'
        organizations[2][0] = u'1'
        filehandle = StringIO()
        self.create_test_workbook(filehandle, organizations=organizations)
        xls_file = filehandle.getvalue()
        filehandle.close()

        browser.open('/import-agencies')
        widget = browser.getControl(name='form.widgets.xls_file')
        widget.add_file(xls_file, 'application/xls', 'import.xls')
        browser.getControl('Import').click()
        self.assertIn('Top level organizations not found', browser.contents)
        self.assertIn('Invalid row 3 in sheet Organizations', browser.contents)
